from os import getenv

from aiogram import Bot, F, Router, html
from aiogram.fsm.context import FSMContext
from aiogram.fsm.state import State, StatesGroup
from aiogram.types import (
    CallbackQuery,
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    KeyboardButton,
    Message,
    ReplyKeyboardMarkup,
    ReplyKeyboardRemove,
)
from aiogram.utils.formatting import Bold, as_key_value, as_list, as_marked_section
from pyquizAPI import QuizClient


async def get_question():
    api = getenv('QUIZ_API')
    client = QuizClient(api)
    client.make_config(limit=1, difficulty='Easy')

    questions = client.get_questions(use_config=True)
    return questions


router = Router()


class GameState(StatesGroup):
    quiz = State()


@router.message(F.text.lower() == 'exit game')
async def exit_games(message: Message, state: FSMContext):
    await state.clear()
    await message.reply('Game is finished', reply_markup=ReplyKeyboardRemove())


async def get_formated_question(
    qnt: int, correct: int, question: str, answers: list[str]
):
    content = as_list(
        as_marked_section(
            Bold('Game summary:'),
            as_key_value('Total questions', qnt),
            as_key_value('Success', correct),
            as_key_value('Failed', qnt - correct),
            marker='  ',
        ),
        as_marked_section(
            Bold('Question:'),
            html.quote(question),
            marker='❔ ',
        ),
        as_marked_section(
            Bold('Answers:'),
            *[html.quote(item) for item in answers],
            marker='➖ ',
        ),
        sep='\n\n',
    )
    return content


@router.message(F.text.lower() == 'quiz')
async def start_quiz(message: Message, state: FSMContext):
    quiz_kb = [
        [KeyboardButton(text='Next question')],
        [KeyboardButton(text='Exit Game')],
    ]
    await state.set_state(GameState.quiz)

    await message.reply('Start Quiz', reply_markup=ReplyKeyboardRemove())
    await message.reply(
        'First question:',
        reply_markup=ReplyKeyboardMarkup(keyboard=quiz_kb, resize_keyboard=True),
    )
    await question(message, state)


@router.message(GameState.quiz, F.text)
async def question(message: Message, state: FSMContext):
    data = await state.get_data()
    question_data = await get_question()
    question = question_data[0].get('question')
    answers = {
        question_data[0]['answers'][k]: question_data[0]['correct_answers'][
            k + '_correct'
        ]
        for k in question_data[0]['answers'].keys()
        if question_data[0]['answers'][k] is not None
    }
    buttons = []

    idx = 1

    for k, v in answers.items():
        buttons.append([InlineKeyboardButton(text=f'choice {idx}', callback_data=k)])
        if v == 'true':
            data['answer'] = k
        idx += 1

    content = await get_formated_question(
        data.get('quantity', 0), data.get('score', 0), question, list(answers.keys())
    )
    data['question'] = question
    await state.update_data(data)

    await message.answer(
        **content.as_kwargs(),
        reply_markup=InlineKeyboardMarkup(inline_keyboard=buttons),
    )


@router.callback_query(GameState.quiz)
async def check_answer(callback: CallbackQuery, state: FSMContext, bot: Bot):
    data = await state.get_data()
    answer = data['answer']
    score = data.get('score', 0)
    qnt = data.get('quantity', 0)

    await callback.answer()
    if answer == callback.data:
        await callback.message.answer(text='Correct')
        score += 1
    else:
        await callback.message.answer(text='Opps. You miss')
    qnt += 1

    data['score'] = score
    data['quantity'] = qnt
    await state.update_data(data)

    msg_text: str = callback.message.text

    await bot.edit_message_text(
        chat_id=callback.from_user.id,
        message_id=callback.message.message_id,
        text=msg_text,
    )
